import minimalmodbus
import serial
from datetime import datetime
import schedule
import time
import os
import sys
import linecache

# I think these are needed to keep things running smoothly...
os.fork() and sys.exit()

# set up the wired controller connection
instrument = minimalmodbus.Instrument('/dev/ttyUSB0', 1)  
instrument.serial.port  
instrument.serial.baudrate = 9600         
instrument.serial.bytesize = 8
instrument.serial.parity   = serial.PARITY_NONE
instrument.serial.stopbits = 1
instrument.mode = minimalmodbus.MODE_RTU

# set up the aIAT sensor connection (uses the same serial settings as above)
instrument_MD02 = minimalmodbus.Instrument('/dev/ttyUSB0', 3)
instrument_MD02.mode = minimalmodbus.MODE_RTU

# set the desired IAT
dIAT = 19

def hrCheck():

    # check DHW is NOT running (use three port two way valve position) 

    on_off = instrument.read_register(129, 0)
    valve_pos = "{:08b}".format(on_off)[-5]      # 1 = DHW is on, 0 = DHW is off

    # note valve_pos is a string, not an integer, hence need '0' not 0
    if valve_pos == '0':

        # get a datetime for logging
        now = datetime.now()
        now = now.strftime("%Y-%m-%dT%H:%M:%S")

        # get the aIAT (it is reported as x 10, ie 12.6 as 126 so need to divide by 10)
        aIAT = instrument_MD02.read_register(1, 0, 4)/10

        #sleep a while to reduce bus collisions
        time.sleep(0.1)    

        # compare the aIAT to the dIAT and set the ends of the WCC accordingly
        # for now, keep it simple, no loops, write code out in full

        # set the baseline, non-adapted curve values
        # you can change these anytime, edit and save the script and then re-start it

        leftBase = 53
        rightBase = 31
        
        if aIAT - dIAT >= 3:     # house is too warm, lower ends of WCC by 3 degree C
            instrument.write_register(265, leftBase - 3, 0)  
            instrument.write_register(266, rightBase - 3, 0)
            # log the current actual values after a short pause
            time.sleep(0.1)
            leftWCC = instrument.read_register(265, 0)
            rightWCC = instrument.read_register(266, 0)
            with open('./modbus/auto_adapt_log.csv', 'a') as f:
                f.write(f'\n{now},{round(aIAT - dIAT, 1)},{leftWCC},{rightWCC}')

        elif aIAT - dIAT >= 2 and aIAT - dIAT < 3:     # house is too warm, lower ends of WCC by 2 degree C
            instrument.write_register(265, leftBase - 2, 0)  
            instrument.write_register(266, rightBase - 2, 0)
            # log the current actual values after a short pause
            time.sleep(0.1)
            leftWCC = instrument.read_register(265, 0)
            rightWCC = instrument.read_register(266, 0)
            with open('./modbus/auto_adapt_log.csv', 'a') as f:
                f.write(f'\n{now},{round(aIAT - dIAT, 1)},{leftWCC},{rightWCC}')

        elif aIAT - dIAT >= 1 and aIAT - dIAT < 2:     # house is too warm, lower ends of WCC by 1 degree C
            instrument.write_register(265, leftBase - 1, 0)  
            instrument.write_register(266, rightBase - 1, 0)
            # log the current actual values after a short pause
            time.sleep(0.1)
            leftWCC = instrument.read_register(265, 0)
            rightWCC = instrument.read_register(266, 0)
            with open('./modbus/auto_adapt_log.csv', 'a') as f:
                f.write(f'\n{now},{round(aIAT - dIAT, 1)},{leftWCC},{rightWCC}')

        elif aIAT - dIAT > -1 and aIAT - dIAT < 1:     # house is right range, set non-adapted WCC ends
            instrument.write_register(265, leftBase, 0)  
            instrument.write_register(266, rightBase, 0)
            # log the current actual values after a short pause
            time.sleep(0.1)
            leftWCC = instrument.read_register(265, 0)
            rightWCC = instrument.read_register(266, 0)
            with open('./modbus/auto_adapt_log.csv', 'a') as f:
                f.write(f'\n{now},{round(aIAT - dIAT, 1)},{leftWCC},{rightWCC}')

        elif aIAT - dIAT > -2 and aIAT - dIAT <= -1:     # house is too cool, raise ends of WCC by 1 degree C
            instrument.write_register(265, leftBase + 1, 0)  
            instrument.write_register(266, rightBase + 1, 0)
            # log the current actual values after a short pause
            time.sleep(0.1)
            leftWCC = instrument.read_register(265, 0)
            rightWCC = instrument.read_register(266, 0)
            with open('./modbus/auto_adapt_log.csv', 'a') as f:
                f.write(f'\n{now},{round(aIAT - dIAT, 1)},{leftWCC},{rightWCC}')

        elif aIAT - dIAT > -3 and aIAT - dIAT <= -2:     # house is too cool, raise ends of WCC by 2 degree C
            instrument.write_register(265, leftBase + 2, 0)  
            instrument.write_register(266, rightBase + 2, 0)
            # log the current actual values after a short pause
            time.sleep(0.1)
            leftWCC = instrument.read_register(265, 0)
            rightWCC = instrument.read_register(266, 0)
            with open('./modbus/auto_adapt_log.csv', 'a') as f:
                f.write(f'\n{now},{round(aIAT - dIAT, 1)},{leftWCC},{rightWCC}')

        elif aIAT - dIAT <= -3:     # house is too cool, raise ends of WCC by 3 degree C
            instrument.write_register(265, leftBase + 3, 0)  
            instrument.write_register(266, rightBase + 3, 0)
            # log the current actual values after a short pause
            time.sleep(0.1)
            leftWCC = instrument.read_register(265, 0)
            rightWCC = instrument.read_register(266, 0)
            with open('./modbus/auto_adapt_log.csv', 'a') as f:
                f.write(f'\n{now},{round(aIAT - dIAT, 1)},{leftWCC},{rightWCC}')

        else:  # only gets executed if nothing matches ie an error has occured
            # do nothing could also use var = var eg aIAT = aIAT  
            WCCerror = '***error, WCC ends were not set***'
            # log the current actual values after a short pause
            time.sleep(0.1)
            leftWCC = instrument.read_register(265, 0)
            rightWCC = instrument.read_register(266, 0)
            with open('./modbus/auto_adapt_log.csv', 'a') as f:
                f.write(f'\n{now},{round(aIAT - dIAT, 1)},{leftWCC},{rightWCC}')


# do the check every hour at 10m 30s past the hour (won't accept '00' for minutes)
schedule.every().hour.at("10:30").do(hrCheck)

# for testing, uncomment next line and kill/restart in terminal to get one off run
#schedule.run_all()

while True:
    schedule.run_pending()
    time.sleep(1)
